@@ -25,9 +25,7 @@ def test_check_basic_route_url_port(nginx):
 
     (_, _, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local",
-        {},
     )
 
     assert matched_backend == {**nginx, "port": 80}
@@ -44,9 +42,7 @@ def test_check_basic_route_backend_port(nginx):
 
     (_, _, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local:8910",
-        {},
     )
 
     assert matched_backend == {**nginx, "port": 8888}
@@ -66,9 +62,7 @@ def test_hostname_match(nginx):
     with pytest.raises(RuntimeError, match="no route matched"):
         junction.check_route(
             [route],
-            "GET",
             "http://something-else.default.svc.cluster.local",
-            {},
         )
 
 
@@ -86,26 +80,20 @@ def test_check_basic_route_ports_match(nginx):
     with pytest.raises(RuntimeError, match="no route matched"):
         (_, _, matched_backend) = junction.check_route(
             [route],
-            "GET",
             "http://nginx.default.svc.cluster.local:5678",
-            {},
         )
 
     # using the default http/https port should not work
     with pytest.raises(RuntimeError, match="no route matched"):
         (_, _, matched_backend) = junction.check_route(
             [route],
-            "GET",
             "https://nginx.default.svc.cluster.local",
-            {},
         )
 
     # explicitly specifying the right port should work
     (_, _, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local:1234",
-        {},
     )
     assert matched_backend == {**nginx, "port": 1234}
 
@@ -116,9 +104,7 @@ def test_check_empty_route(nginx):
     with pytest.raises(RuntimeError, match="no route matched"):
         _ = junction.check_route(
             [route],
-            "GET",
             "http://nginx.default.svc.cluster.local:1234",
-            {},
         )
 
 
@@ -142,16 +128,12 @@ def test_check_empty_rules(nginx, nginx_staging):
     with pytest.raises(RuntimeError, match="invalid route"):
         _ = junction.check_route(
             [route],
-            "GET",
             "http://nginx.default.svc.cluster.local:1234",
-            {},
         )
 
     (_, matching_rule_idx, backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local:1234/users",
-        {},
     )
     assert matching_rule_idx == 0
     assert {**nginx_staging, "port": 8910} == backend
@@ -176,9 +158,7 @@ def test_check_retry_and_timeouts(nginx):
 
     (matching_route, matching_rule_idx, _) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local",
-        {},
     )
 
     assert timeouts == matching_route["rules"][matching_rule_idx]["timeouts"]
@@ -194,9 +174,7 @@ def test_check_redirect_route(nginx_staging):
 
     (_, _, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local",
-        {},
     )
 
     assert matched_backend == {**nginx_staging, "port": 80}
@@ -226,15 +204,44 @@ def test_check_headers_matches_default(headers, nginx, nginx_staging):
     # match without headers
     (_, matched_rule_idx, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local",
-        headers,
+        headers=headers,
     )
 
     assert {**nginx, "port": 80} == matched_backend
     assert [{**nginx, "port": 80}] == route["rules"][matched_rule_idx]["backends"]
 
 
+def test_check_method_match(nginx, nginx_staging):
+    route: config.Route = {
+        "id": "test-route",
+        "hostnames": ["nginx.default.svc.cluster.local"],
+        "rules": [
+            {
+                "backends": [{**nginx_staging, "port": 80}],
+                "matches": [{"method": "POST"}],
+            },
+            {"backends": [{**nginx, "port": 80}]},
+        ],
+    }
+
+    # match a POST
+    (_, _, matched_backend) = junction.check_route(
+        [route],
+        "http://nginx.default.svc.cluster.local",
+        method="POST",
+    )
+    assert {**nginx_staging, "port": 80} == matched_backend
+
+    # don't match a PUT
+    (_, _, matched_backend) = junction.check_route(
+        [route],
+        "http://nginx.default.svc.cluster.local",
+        method="PUT",
+    )
+    assert {**nginx, "port": 80} == matched_backend
+
+
 def test_check_headers_match(nginx, nginx_staging):
     route: config.Route = {
         "id": "test-route",
@@ -251,9 +258,8 @@ def test_check_headers_match(nginx, nginx_staging):
     # match without headers
     (_, matched_rule_idx, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local",
-        {"x-env": "staging"},
+        headers={"x-env": "staging"},
     )
 
     assert {**nginx_staging, "port": 80} == matched_backend
@@ -279,9 +285,7 @@ def test_check_path_matches_default(nginx, nginx_staging):
 
     (_, matched_rule_idx, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local/barfoo",
-        {},
     )
 
     assert {**nginx, "port": 80} == matched_backend
@@ -305,9 +309,7 @@ def test_check_path_matches(nginx, nginx_staging):
 
     (_, matched_rule_idx, matched_backend) = junction.check_route(
         [route],
-        "GET",
         "http://nginx.default.svc.cluster.local/fooblahbar",
-        {},
     )
 
     assert {**nginx_staging, "port": 80} == matched_backend
